@@ -9,7 +9,7 @@ This module inherits constants for Denon AVR receivers.
 
 import re
 
-from collections import namedtuple, OrderedDict
+from collections import namedtuple
 
 import attr
 
@@ -62,54 +62,61 @@ CHANGE_INPUT_MAPPING = {"Internet Radio": "IRP", "Online Music": "NET",
                         "Media Server": "SERVER", "Spotify": "SPOTIFY",
                         "Flickr": "FLICKR", "Favorites": "FAVORITES"}
 
+TELNET_SOURCES = ["CD", "PHONO", "TUNER", "DVD", "BD", "TV",
+                  "SAT/CBL", "MPLAY", "GAME", "HDRADIO", "NET", "PANDORA",
+                  "SIRIUSXM", "LASTFM", "FLICKR", "IRADIO", "SERVER",
+                  "FAVORITES", "AUX1", "AUX2", "AUX3", "AUX4", "AUX5",
+                  "AUX6", "AUX7", "BT", "USB/IPOD", "USB DIRECT",
+                  "IPOD DIRECT"]
+
 # Sound modes
 ALL_ZONE_STEREO = "ALL ZONE STEREO"
 
-SOUND_MODE_MAPPING = OrderedDict(
-    [("MUSIC", ["PLII MUSIC", "DTS NEO:6 MUSIC", "DOLBY D +NEO:X M",
-                "DTS NEO:X MUSIC", "DOLBY PL2 MUSIC", "PLIIX MUSIC"]),
-     ("MOVIE", ["PLII MOVIE", "PLII CINEMA", "DTS NEO:X CINEMA",
-                "DTS NEO:6 CINEMA", "DOLBY D +NEO:X C",
-                "PLIIX CINEMA", "DOLBY PLII MOVIE", "MULTI IN + VIRTUAL:X",
-                "DOLBY PL2 CINEMA"]),
-     ("GAME", ["PLII GAME", "DOLBY D +NEO:X G"]),
-     ("AUTO", ["None"]),
-     ("STANDARD", ["None2"]),
-     ("VIRTUAL", ["VIRTUAL"]),
-     ("MATRIX", ["MATRIX"]),
-     ("ROCK ARENA", ["ROCK ARENA"]),
-     ("JAZZ CLUB", ["JAZZ CLUB"]),
-     ("VIDEO GAME", ["VIDEO GAME"]),
-     ("MONO MOVIE", ["MONO MOVIE"]),
-     ("DIRECT", ["DIRECT"]),
-     ("PURE DIRECT", ["PURE_DIRECT", "PURE DIRECT"]),
-     ("DOLBY DIGITAL", ["DOLBY DIGITAL", "DOLBY D + DOLBY SURROUND",
-                        "DOLBY DIGITAL +", "STANDARD(DOLBY)", "DOLBY SURROUND",
-                        "DOLBY D + +DOLBY SURROUND", "NEURAL", "DOLBY HD",
-                        "DOLBY HD + DOLBY SURROUND",
-                        "MULTI IN + NEURAL:X", "MULTI IN + DOLBY SURROUND",
-                        "DOLBY D + NEURAL:X", "DOLBY DIGITAL + NEURAL:X",
-                        "DOLBY DIGITAL + + NEURAL:X", "DOLBY ATMOS",
-                        "DOLBY AUDIO - DOLBY SURROUND", "DOLBY TRUEHD",
-                        "DOLBY AUDIO - DOLBY DIGITAL PLUS",
-                        "DOLBY AUDIO - TRUEHD + DSUR",
-                        "DOLBY AUDIO - DOLBY TRUEHD",
-                        "DOLBY AUDIO - TRUEHD + NEURAL:X",
-                        "DOLBY AUDIO - DD + DSUR",
-                        "DOLBY AUDIO - DD+   + NEURAL:X",
-                        "DOLBY AUDIO - DD+   + DSUR",
-                        "DOLBY AUDIO - DOLBY DIGITAL"]),
-     ("DTS SURROUND", ["DTS SURROUND", "DTS NEURAL:X", "STANDARD(DTS)",
-                       "DTS + NEURAL:X", "MULTI CH IN", "DTS-HD MSTR",
-                       "DTS VIRTUAL:X", "DTS-HD + NEURAL:X", "DTS-HD",
-                       "DTS + VIRTUAL:X", "DTS + DOLBY SURROUND",
-                       "DTS-HD + DOLBY SURROUND", "DTS-HD + DSUR",
-                       "DTS:X MSTR"]),
-     ("AURO3D", ["AURO-3D", "AURO-2D SURROUND"]),
-     ("MCH STEREO", ["MULTI CH STEREO", "MULTI_CH_STEREO", "MCH STEREO",
-                     "MULTI CH IN 7.1"]),
-     ("STEREO", ["STEREO"]),
-     (ALL_ZONE_STEREO, ["ALL ZONE STEREO"])])
+SOUND_MODE_MAPPING = {
+    "MUSIC": ["PLII MUSIC", "DTS NEO:6 MUSIC", "DOLBY D +NEO:X M",
+              "DTS NEO:X MUSIC", "DOLBY PL2 MUSIC", "PLIIX MUSIC"],
+    "MOVIE": ["PLII MOVIE", "PLII CINEMA", "DTS NEO:X CINEMA",
+              "DTS NEO:6 CINEMA", "DOLBY D +NEO:X C",
+              "PLIIX CINEMA", "DOLBY PLII MOVIE", "MULTI IN + VIRTUAL:X",
+              "DOLBY PL2 CINEMA"],
+    "GAME": ["PLII GAME", "DOLBY D +NEO:X G", "DOLBY PL2 GAME"],
+    "AUTO": ["None"],
+    "STANDARD": ["None2"],
+    "VIRTUAL": ["VIRTUAL"],
+    "MATRIX": ["MATRIX"],
+    "ROCK ARENA": ["ROCK ARENA"],
+    "JAZZ CLUB": ["JAZZ CLUB"],
+    "VIDEO GAME": ["VIDEO GAME"],
+    "MONO MOVIE": ["MONO MOVIE"],
+    "DIRECT": ["DIRECT"],
+    "PURE DIRECT": ["PURE_DIRECT", "PURE DIRECT"],
+    "DOLBY DIGITAL": ["DOLBY DIGITAL", "DOLBY D + DOLBY SURROUND",
+                      "DOLBY DIGITAL +", "STANDARD(DOLBY)", "DOLBY SURROUND",
+                      "DOLBY D + +DOLBY SURROUND", "NEURAL", "DOLBY HD",
+                      "DOLBY HD + DOLBY SURROUND", "MULTI IN + DSUR",
+                      "MULTI IN + NEURAL:X", "MULTI IN + DOLBY SURROUND",
+                      "DOLBY D + NEURAL:X", "DOLBY DIGITAL + NEURAL:X",
+                      "DOLBY DIGITAL + + NEURAL:X", "DOLBY ATMOS",
+                      "DOLBY AUDIO - DOLBY SURROUND", "DOLBY TRUEHD",
+                      "DOLBY AUDIO - DOLBY DIGITAL PLUS",
+                      "DOLBY AUDIO - TRUEHD + DSUR",
+                      "DOLBY AUDIO - DOLBY TRUEHD",
+                      "DOLBY AUDIO - TRUEHD + NEURAL:X",
+                      "DOLBY AUDIO - DD + DSUR",
+                      "DOLBY AUDIO - DD+   + NEURAL:X",
+                      "DOLBY AUDIO - DD+   + DSUR",
+                      "DOLBY AUDIO - DOLBY DIGITAL"],
+    "DTS SURROUND": ["DTS SURROUND", "DTS NEURAL:X", "STANDARD(DTS)",
+                     "DTS + NEURAL:X", "MULTI CH IN", "DTS-HD MSTR",
+                     "DTS VIRTUAL:X", "DTS-HD + NEURAL:X", "DTS-HD",
+                     "DTS + VIRTUAL:X", "DTS + DOLBY SURROUND",
+                     "DTS-HD + DOLBY SURROUND", "DTS-HD + DSUR",
+                     "DTS:X MSTR"],
+    "AURO3D": ["AURO-3D", "AURO-2D SURROUND"],
+    "MCH STEREO": ["MULTI CH STEREO", "MULTI_CH_STEREO", "MCH STEREO",
+                   "MULTI CH IN 7.1"],
+    "STEREO": ["STEREO"],
+    ALL_ZONE_STEREO: ["ALL ZONE STEREO"]}
 
 # Receiver sources
 PLAYING_SOURCES = (
@@ -248,6 +255,9 @@ ZONE3_URLS = ReceiverURLs(
     command_pause=COMMAND_PAUSE,
     command_play=COMMAND_PLAY)
 
+# Telnet Commands
+TELNET_EVENTS = ["PW", "MV", "MU", "SI", "MS", "PS"]
+
 # States
 POWER_ON = "ON"
 POWER_OFF = "OFF"
