@@ -7,6 +7,7 @@ This module implements the foundation classes for Denon AVR receivers.
 :license: MIT, see LICENSE for more details.
 """
 
+import asyncio
 import logging
 import xml.etree.ElementTree as ET
 
@@ -17,10 +18,10 @@ import attr
 import httpx
 
 from .appcommand import AppCommandCmd, AppCommands
-from .api import DenonAVRApi
+from .api import DenonAVRApi, DenonAVRTelnetApi
 from .exceptions import (
-    AvrForbiddenError, AvrNetworkError, AvrProcessingError, AvrRequestError,
-    AvrTimoutError)
+    AvrForbiddenError, AvrIncompleteResponseError, AvrNetworkError,
+    AvrProcessingError, AvrRequestError, AvrTimoutError)
 from .const import (
     APPCOMMAND_CMD_TEXT, APPCOMMAND_NAME, AVR, AVR_X, AVR_X_2016,
     DENON_ATTR_SETATTR, DENONAVR_URLS, DESCRIPTION_TYPES,
@@ -41,6 +42,11 @@ class DenonAVRDeviceInfo:
         validator=attr.validators.instance_of(DenonAVRApi),
         default=attr.Factory(DenonAVRApi),
         kw_only=True)
+    telnet_api: DenonAVRTelnetApi = attr.ib(
+        validator=attr.validators.instance_of(DenonAVRTelnetApi),
+        default=attr.Factory(DenonAVRTelnetApi),
+        kw_only=True
+    )
     receiver: Optional[ReceiverType] = attr.ib(
         validator=attr.validators.optional(
             attr.validators.in_(VALID_RECEIVER_TYPES)),
@@ -67,9 +73,10 @@ class DenonAVRDeviceInfo:
     _is_setup: bool = attr.ib(converter=bool, default=False, init=False)
     _allow_recovery: bool = attr.ib(
         converter=bool, default=False, init=False)
+    _setup_lock: asyncio.Lock = attr.ib(default=attr.Factory(asyncio.Lock))
 
     def __attrs_post_init__(self) -> None:
-        """Initialize special attributes."""
+        """Initialize special attributes and callbacks."""
         # URLs depending from value of self.zone attribute
         if self.zone == MAIN_ZONE:
             self.urls = DENONAVR_URLS
@@ -80,6 +87,15 @@ class DenonAVRDeviceInfo:
         else:
             raise ValueError("Invalid zone {}".format(self.zone))
 
+    async def _power_callback(
+            self,
+            zone: str,
+            event: str,
+            parameter: str) -> None:
+        """Handle a power change event."""
+        if self.zone == zone:
+            self._power = parameter
+
     def get_own_zone(self):
         """
         Get zone from actual instance.
@@ -93,15 +109,26 @@ class DenonAVRDeviceInfo:
 
     async def async_setup(self) -> None:
         """Ensure that configuration is loaded from receiver asynchronously."""
-        # Own setup
-        await self.async_identify_receiver()
-        await self.async_get_device_info()
-        await self.async_identify_update_method()
+        async with self._setup_lock:
+            # Own setup
+            # Reduce read timeout during receiver identification
+            # deviceinfo endpoint takes very long to return 404
+            timeout = self.api.timeout
+            self.api.timeout = httpx.Timeout(self.api.timeout.connect)
+            try:
+                await self.async_identify_receiver()
+                await self.async_get_device_info()
+            finally:
+                self.api.timeout = timeout
+            await self.async_identify_update_method()
 
-        # Add tags for a potential AppCommand.xml update
-        self.api.add_appcommand_update_tag(AppCommands.GetAllZonePowerStatus)
+            # Add tags for a potential AppCommand.xml update
+            self.api.add_appcommand_update_tag(
+                AppCommands.GetAllZonePowerStatus)
 
-        self._is_setup = True
+            self.telnet_api.register_callback("PW", self._power_callback)
+
+            self._is_setup = True
 
     async def async_update(
             self,
@@ -237,46 +264,48 @@ class DenonAVRDeviceInfo:
 
     async def async_verify_avr_2016_update_method(
             self, cache_id: Hashable = None) -> None:
-        """Verify if avr 2016 update method is working with all tags."""
-        # AVR receivers do not support AppCommand.xml interface
-        if self.receiver == AVR:
-            self.use_avr_2016_update = False
-        else:
-            try:
-                await self.api.async_get_global_appcommand(cache_id=cache_id)
-            except (AvrTimoutError, AvrNetworkError) as err:
-                _LOGGER.debug(
-                    "Connection error when verifying update method",
-                    exc_info=err)
-                raise
-            except AvrForbiddenError:
-                # Recovery in case receiver changes port from 80 to 8080 which
-                # might happen at Denon AVR-X 2016 receivers
-                if self._allow_recovery is True:
-                    self._allow_recovery = False
-                    _LOGGER.warning(
-                        "AppCommand.xml returns HTTP status 403. Running setup"
-                        " again once to check if receiver interface switched "
-                        "ports")
-                    self._is_setup = False
-                    await self.async_setup()
-                    await self.async_verify_avr_2016_update_method(
-                        cache_id=cache_id)
-                else:
-                    raise
-            except AvrRequestError as err:
-                _LOGGER.debug(
-                    "Request error when verifying update method", exc_info=err)
-                # Only AVR_X devices support both interfaces
-                if self.receiver == AVR_X:
-                    if self.use_avr_2016_update is True:
-                        _LOGGER.warning(
-                            "Error verifying Appcommand.xml update method, "
-                            "deactivate this interface")
-                    self.use_avr_2016_update = False
+        """Verify if avr 2016 update method is working."""
+        # Nothing to do if Appcommand.xml interface is not supported
+        if self.use_avr_2016_update is False:
+            return
+
+        try:
+            # Result is cached that it can be reused during update
+            await self.api.async_get_global_appcommand(cache_id=cache_id)
+        except (AvrTimoutError, AvrNetworkError) as err:
+            _LOGGER.debug(
+                "Connection error when verifying update method",
+                exc_info=err)
+            raise
+        except AvrForbiddenError:
+            # Recovery in case receiver changes port from 80 to 8080 which
+            # might happen at Denon AVR-X 2016 receivers
+            if self._allow_recovery is True:
+                self._allow_recovery = False
+                _LOGGER.warning(
+                    "AppCommand.xml returns HTTP status 403. Running setup"
+                    " again once to check if receiver interface switched "
+                    "ports")
+                self._is_setup = False
+                await self.async_setup()
+                await self.async_verify_avr_2016_update_method(
+                    cache_id=cache_id)
             else:
-                self._allow_recovery = True
-                self.use_avr_2016_update = True
+                raise
+        except AvrIncompleteResponseError as err:
+            _LOGGER.debug(
+                "Request error when verifying update method", exc_info=err)
+            # Only AVR_X devices support both interfaces
+            if self.receiver == AVR_X:
+                _LOGGER.warning(
+                    "Error verifying Appcommand.xml update method, it returns "
+                    "an incomplete result set. Deactivating the interface")
+                self.use_avr_2016_update = False
+        else:
+            if self._allow_recovery is False:
+                _LOGGER.info(
+                    "AppCommand.xml recovered from HTTP status 403 error")
+            self._allow_recovery = True
 
     def _set_friendly_name(self, xml: ET.Element) -> None:
         """Set FriendlyName from result xml."""
@@ -355,17 +384,15 @@ class DenonAVRDeviceInfo:
             global_update: bool = False,
             cache_id: Optional[Hashable] = None):
         """Update power status from AppCommand.xml."""
-        # Collect tags for AppCommand.xml call
-        update_attrs = {"_power": AppCommands.GetAllZonePowerStatus}
-        tags = tuple(i for i in update_attrs.values())
-        # Execute call
+        power_appcommand = AppCommands.GetAllZonePowerStatus
         try:
             if global_update:
                 xml = await self.api.async_get_global_appcommand(
                     cache_id=cache_id)
             else:
                 xml = await self.api.async_post_appcommand(
-                    self.urls.appcommand, tags, cache_id=cache_id)
+                    self.urls.appcommand, tuple(power_appcommand),
+                    cache_id=cache_id)
         except AvrRequestError as err:
             _LOGGER.debug(
                 "Error when getting power status", exc_info=err)
@@ -374,32 +401,18 @@ class DenonAVRDeviceInfo:
         # Extract relevant information
         zone = self.get_own_zone()
 
-        attrs = deepcopy(update_attrs)
-        for name, tag in attrs.items():
-            try:
-                # Check if attribute exists
-                getattr(self, name)
-                # Set new value
-                setattr(
-                    self,
-                    name,
-                    xml.find("./cmd[@{attribute}='{cmd}']/{zone}".format(
-                        attribute=APPCOMMAND_CMD_TEXT,
-                        cmd=tag.cmd_text,
-                        zone=zone)).text)
-                # Done
-                update_attrs.pop(name, None)
+        # Search for power tag
+        power_tag = xml.find("./cmd[@{attribute}='{cmd}']/{zone}".format(
+            attribute=APPCOMMAND_CMD_TEXT,
+            cmd=power_appcommand.cmd_text,
+            zone=zone))
 
-            except (AttributeError, IndexError) as err:
-                _LOGGER.debug(
-                    "Failed updating attribute %s for zone %s", name,
-                    self.zone, exc_info=err)
-
-        # Check if each attribute was updated
-        if update_attrs:
+        if power_tag is None:
             raise AvrProcessingError(
-                "Not all attributes updated, those are left: {}".format(
-                    update_attrs))
+                "Power attribute of zone {} not found on update".format(
+                    self.zone))
+
+        self._power = power_tag.text
 
     async def async_update_power_status_xml(
             self,
@@ -409,41 +422,31 @@ class DenonAVRDeviceInfo:
         urls = [self.urls.status]
         if self.zone == MAIN_ZONE:
             urls.append(self.urls.mainzone)
-        # Variables with their tags to be updated
-        update_attrs = {"_power": "./Power/value"}
+        else:
+            urls.append("{}?ZoneName={}".format(self.urls.mainzone, self.zone))
+        # Tags in XML which might contain information about zones power status
+        # ordered by their priority
+        tags = ["./ZonePower/value", "./Power/value"]
 
-        for url in urls:
-            try:
-                xml = await self.api.async_get_xml(
-                    url, cache_id=cache_id)
-            except AvrRequestError as err:
-                _LOGGER.debug(
-                    "Error when getting power status from url %s", url,
-                    exc_info=err)
-                continue
-            attrs = deepcopy(update_attrs)
-            for name, tag in attrs.items():
+        for tag in tags:
+            for url in urls:
                 try:
-                    # Check if attribute exists
-                    getattr(self, name)
-                    # Set new value
-                    setattr(self, name, xml.find(tag).text)
-                    # Done
-                    update_attrs.pop(name, None)
-                except (AttributeError, IndexError) as err:
+                    xml = await self.api.async_get_xml(
+                        url, cache_id=cache_id)
+                except AvrRequestError as err:
                     _LOGGER.debug(
-                        "Failed updating attribute %s for zone %s", name,
-                        self.zone, exc_info=err)
+                        "Error when getting power status from url %s", url,
+                        exc_info=err)
+                    continue
 
-            # All done, no need for continuing
-            if not update_attrs:
-                break
+                # Search for power tag
+                power_tag = xml.find(tag)
+                if power_tag is not None and power_tag.text is not None:
+                    self._power = power_tag.text
+                    return
 
-        # Check if each attribute was updated
-        if update_attrs:
-            raise AvrProcessingError(
-                "Not all attributes updated, those are left: {}".format(
-                    update_attrs))
+        raise AvrProcessingError(
+            "Power attribute of zone {} not found on update".format(self.zone))
 
     ##############
     # Properties #
@@ -559,12 +562,12 @@ class DenonAVRFoundation:
         # Check if each attribute was updated
         if update_attrs and ignore_missing_response is False:
             raise AvrProcessingError(
-                "Not all attributes updated, those are left: {}".format(
-                    update_attrs))
+                "Some attributes of zone {} not found on update: {}".format(
+                    self._device.zone, update_attrs))
         if update_attrs and ignore_missing_response is True:
             _LOGGER.debug(
-                "Not all attributes updated, those are left and ignored "
-                "deliberately: %s", update_attrs)
+                "Some attributes of zone %s not found on update: %s",
+                self._device.zone, update_attrs)
 
     async def async_update_attrs_status_xml(
             self,
@@ -620,8 +623,8 @@ class DenonAVRFoundation:
         # Check if each attribute was updated
         if update_attrs and ignore_missing_response is False:
             raise AvrProcessingError(
-                "Not all attributes updated, those are left: {}".format(
-                    update_attrs))
+                "Some attributes of zone {} not found on update: {}".format(
+                    self._device.zone, update_attrs))
 
     @staticmethod
     def create_appcommand_search_strings(
@@ -658,7 +661,9 @@ def set_api_host(
         value: str) -> str:
     """Change API host on host changes too."""
     # First change _device.api.host then return value
-    instance._device.api.host = value  # pylint: disable=protected-access
+    # pylint: disable=protected-access
+    instance._device.api.host = value
+    instance._device.telnet_api.host = value
     return value
 
 
@@ -669,7 +674,9 @@ def set_api_timeout(
     """Change API timeout on timeout changes too."""
     # First change _device.api.host then return value
     timeout = httpx.Timeout(value, read=max(value, 15.0))
-    instance._device.api.timeout = timeout  # pylint: disable=protected-access
+    # pylint: disable=protected-access
+    instance._device.api.timeout = timeout
+    instance._device.telnet_api.timeout = value
     return value
 
 
